# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["SessionGetSessionParticipantsParams"]


class SessionGetSessionParticipantsParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    app_id: Required[str]
    """The app identifier tag."""

    include_peer_events: bool
    """if true, response includes all the peer events of participants."""

    page_no: float
    """The page number from which you want your page search results to be displayed."""

    per_page: float
    """Number of results per page"""

    search: str
    """The search query string. You can search using the meeting ID or title."""

    sort_by: Literal["joinedAt", "duration"]

    sort_order: Literal["ASC", "DESC"]

    view: Literal["raw", "consolidated"]
    """
    In breakout room sessions, the view parameter can be set to `raw` for session
    specific duration for participants or `consolidated` to accumulate breakout room
    durations.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["OpportunisticEncryptionParam"]


class OpportunisticEncryptionParam(TypedDict, total=False):
    id: Literal["opportunistic_encryption"]
    """
    Opportunistic Encryption allows browsers to access HTTP URIs over an encrypted
    TLS channel. It's not a substitute for HTTPS, but provides additional security
    for otherwise vulnerable requests.
    """

    value: Literal["on", "off"]
    """The status of Opportunistic Encryption."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["DEXTestGetResponse", "Data", "TargetPolicy"]


class Data(BaseModel):
    """
    The configuration object which contains the details for the WARP client to conduct the test.
    """

    host: str
    """The desired endpoint to test."""

    kind: Literal["http", "traceroute"]
    """The type of test."""

    method: Optional[Literal["GET"]] = None
    """The HTTP request method type."""


class TargetPolicy(BaseModel):
    id: str
    """API Resource UUID tag."""

    default: Optional[bool] = None
    """Whether the DEX rule is the account default"""

    name: Optional[str] = None
    """The name of the DEX rule"""


class DEXTestGetResponse(BaseModel):
    data: Data
    """
    The configuration object which contains the details for the WARP client to
    conduct the test.
    """

    enabled: bool
    """Determines whether or not the test is active."""

    interval: str
    """How often the test will run."""

    name: str
    """The name of the DEX test. Must be unique."""

    description: Optional[str] = None
    """Additional details about the test."""

    target_policies: Optional[List[TargetPolicy]] = None
    """DEX rules targeted by this test"""

    targeted: Optional[bool] = None

    test_id: Optional[str] = None
    """The unique identifier for the test."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .logging import Logging
from ..._models import BaseModel

__all__ = [
    "ExecuteRule",
    "ActionParameters",
    "ActionParametersMatchedData",
    "ActionParametersOverrides",
    "ActionParametersOverridesCategory",
    "ActionParametersOverridesRule",
    "ExposedCredentialCheck",
    "Ratelimit",
]


class ActionParametersMatchedData(BaseModel):
    """The configuration to use for matched data logging."""

    public_key: str
    """The public key to encrypt matched data logs with."""


class ActionParametersOverridesCategory(BaseModel):
    """A category-level override."""

    category: str
    """The name of the category to override."""

    action: Optional[str] = None
    """The action to override rules in the category with."""

    enabled: Optional[bool] = None
    """Whether to enable execution of rules in the category."""

    sensitivity_level: Optional[Literal["default", "medium", "low", "eoff"]] = None
    """The sensitivity level to use for rules in the category.

    This option is only applicable for DDoS phases.
    """


class ActionParametersOverridesRule(BaseModel):
    """A rule-level override."""

    id: str
    """The ID of the rule to override."""

    action: Optional[str] = None
    """The action to override the rule with."""

    enabled: Optional[bool] = None
    """Whether to enable execution of the rule."""

    score_threshold: Optional[int] = None
    """The score threshold to use for the rule."""

    sensitivity_level: Optional[Literal["default", "medium", "low", "eoff"]] = None
    """The sensitivity level to use for the rule.

    This option is only applicable for DDoS phases.
    """


class ActionParametersOverrides(BaseModel):
    """A set of overrides to apply to the target ruleset."""

    action: Optional[str] = None
    """An action to override all rules with.

    This option has lower precedence than rule and category overrides.
    """

    categories: Optional[List[ActionParametersOverridesCategory]] = None
    """A list of category-level overrides.

    This option has the second-highest precedence after rule-level overrides.
    """

    enabled: Optional[bool] = None
    """Whether to enable execution of all rules.

    This option has lower precedence than rule and category overrides.
    """

    rules: Optional[List[ActionParametersOverridesRule]] = None
    """A list of rule-level overrides. This option has the highest precedence."""

    sensitivity_level: Optional[Literal["default", "medium", "low", "eoff"]] = None
    """A sensitivity level to set for all rules.

    This option has lower precedence than rule and category overrides and is only
    applicable for DDoS phases.
    """


class ActionParameters(BaseModel):
    """The parameters configuring the rule's action."""

    id: str
    """The ID of the ruleset to execute."""

    matched_data: Optional[ActionParametersMatchedData] = None
    """The configuration to use for matched data logging."""

    overrides: Optional[ActionParametersOverrides] = None
    """A set of overrides to apply to the target ruleset."""


class ExposedCredentialCheck(BaseModel):
    """Configuration for exposed credential checking."""

    password_expression: str
    """An expression that selects the password used in the credentials check."""

    username_expression: str
    """An expression that selects the user ID used in the credentials check."""


class Ratelimit(BaseModel):
    """An object configuring the rule's rate limit behavior."""

    characteristics: List[str]
    """
    Characteristics of the request on which the rate limit counter will be
    incremented.
    """

    period: int
    """Period in seconds over which the counter is being incremented."""

    counting_expression: Optional[str] = None
    """An expression that defines when the rate limit counter should be incremented.

    It defaults to the same as the rule's expression.
    """

    mitigation_timeout: Optional[int] = None
    """
    Period of time in seconds after which the action will be disabled following its
    first execution.
    """

    requests_per_period: Optional[int] = None
    """
    The threshold of requests per period after which the action will be executed for
    the first time.
    """

    requests_to_origin: Optional[bool] = None
    """Whether counting is only performed when an origin is reached."""

    score_per_period: Optional[int] = None
    """
    The score threshold per period for which the action will be executed the first
    time.
    """

    score_response_header_name: Optional[str] = None
    """
    A response header name provided by the origin, which contains the score to
    increment rate limit counter with.
    """


class ExecuteRule(BaseModel):
    last_updated: datetime
    """The timestamp of when the rule was last modified."""

    version: str
    """The version of the rule."""

    id: Optional[str] = None
    """The unique ID of the rule."""

    action: Optional[Literal["execute"]] = None
    """The action to perform when the rule matches."""

    action_parameters: Optional[ActionParameters] = None
    """The parameters configuring the rule's action."""

    categories: Optional[List[str]] = None
    """The categories of the rule."""

    description: Optional[str] = None
    """An informative description of the rule."""

    enabled: Optional[bool] = None
    """Whether the rule should be executed."""

    exposed_credential_check: Optional[ExposedCredentialCheck] = None
    """Configuration for exposed credential checking."""

    expression: Optional[str] = None
    """The expression defining which traffic will match the rule."""

    logging: Optional[Logging] = None
    """An object configuring the rule's logging behavior."""

    ratelimit: Optional[Ratelimit] = None
    """An object configuring the rule's rate limit behavior."""

    ref: Optional[str] = None
    """The reference of the rule (the rule's ID by default)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .insights.insights import (
    InsightsResource,
    AsyncInsightsResource,
    InsightsResourceWithRawResponse,
    AsyncInsightsResourceWithRawResponse,
    InsightsResourceWithStreamingResponse,
    AsyncInsightsResourceWithStreamingResponse,
)

__all__ = ["SecurityCenterResource", "AsyncSecurityCenterResource"]


class SecurityCenterResource(SyncAPIResource):
    @cached_property
    def insights(self) -> InsightsResource:
        return InsightsResource(self._client)

    @cached_property
    def with_raw_response(self) -> SecurityCenterResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SecurityCenterResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SecurityCenterResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SecurityCenterResourceWithStreamingResponse(self)


class AsyncSecurityCenterResource(AsyncAPIResource):
    @cached_property
    def insights(self) -> AsyncInsightsResource:
        return AsyncInsightsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSecurityCenterResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSecurityCenterResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSecurityCenterResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSecurityCenterResourceWithStreamingResponse(self)


class SecurityCenterResourceWithRawResponse:
    def __init__(self, security_center: SecurityCenterResource) -> None:
        self._security_center = security_center

    @cached_property
    def insights(self) -> InsightsResourceWithRawResponse:
        return InsightsResourceWithRawResponse(self._security_center.insights)


class AsyncSecurityCenterResourceWithRawResponse:
    def __init__(self, security_center: AsyncSecurityCenterResource) -> None:
        self._security_center = security_center

    @cached_property
    def insights(self) -> AsyncInsightsResourceWithRawResponse:
        return AsyncInsightsResourceWithRawResponse(self._security_center.insights)


class SecurityCenterResourceWithStreamingResponse:
    def __init__(self, security_center: SecurityCenterResource) -> None:
        self._security_center = security_center

    @cached_property
    def insights(self) -> InsightsResourceWithStreamingResponse:
        return InsightsResourceWithStreamingResponse(self._security_center.insights)


class AsyncSecurityCenterResourceWithStreamingResponse:
    def __init__(self, security_center: AsyncSecurityCenterResource) -> None:
        self._security_center = security_center

    @cached_property
    def insights(self) -> AsyncInsightsResourceWithStreamingResponse:
        return AsyncInsightsResourceWithStreamingResponse(self._security_center.insights)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from ...._types import SequenceNotStr

__all__ = ["RegistrationListParams", "Device", "User"]


class RegistrationListParams(TypedDict, total=False):
    account_id: Required[str]

    id: SequenceNotStr[str]
    """Filter by registration ID."""

    cursor: str
    """
    Opaque token indicating the starting position when requesting the next set of
    records. A cursor value can be obtained from the result_info.cursor field in the
    response.
    """

    device: Device

    include: str
    """
    Comma-separated list of additional information that should be included in the
    registration response. Supported values are: "policy".
    """

    per_page: int
    """The maximum number of devices to return in a single response."""

    search: str
    """Filter by registration details."""

    seen_after: str
    """
    Filter by the last_seen timestamp - returns only registrations last seen after
    this timestamp.
    """

    seen_before: str
    """
    Filter by the last_seen timestamp - returns only registrations last seen before
    this timestamp.
    """

    sort_by: Literal["id", "user.name", "user.email", "last_seen_at", "created_at"]
    """The registration field to order results by."""

    sort_order: Literal["asc", "desc"]
    """Sort direction."""

    status: Literal["active", "all", "revoked"]
    """Filter by registration status. Defaults to 'active'."""

    user: User


class Device(TypedDict, total=False):
    id: str
    """Filter by WARP device ID."""


class User(TypedDict, total=False):
    id: SequenceNotStr[str]
    """Filter by user ID."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .usage import (
    UsageResource,
    AsyncUsageResource,
    UsageResourceWithRawResponse,
    AsyncUsageResourceWithRawResponse,
    UsageResourceWithStreamingResponse,
    AsyncUsageResourceWithStreamingResponse,
)
from .billing import (
    BillingResource,
    AsyncBillingResource,
    BillingResourceWithRawResponse,
    AsyncBillingResourceWithRawResponse,
    BillingResourceWithStreamingResponse,
    AsyncBillingResourceWithStreamingResponse,
)
from .profiles import (
    ProfilesResource,
    AsyncProfilesResource,
    ProfilesResourceWithRawResponse,
    AsyncProfilesResourceWithRawResponse,
    ProfilesResourceWithStreamingResponse,
    AsyncProfilesResourceWithStreamingResponse,
)

__all__ = [
    "ProfilesResource",
    "AsyncProfilesResource",
    "ProfilesResourceWithRawResponse",
    "AsyncProfilesResourceWithRawResponse",
    "ProfilesResourceWithStreamingResponse",
    "AsyncProfilesResourceWithStreamingResponse",
    "UsageResource",
    "AsyncUsageResource",
    "UsageResourceWithRawResponse",
    "AsyncUsageResourceWithRawResponse",
    "UsageResourceWithStreamingResponse",
    "AsyncUsageResourceWithStreamingResponse",
    "BillingResource",
    "AsyncBillingResource",
    "BillingResourceWithRawResponse",
    "AsyncBillingResourceWithRawResponse",
    "BillingResourceWithStreamingResponse",
    "AsyncBillingResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from .old_public_schema import OldPublicSchema

__all__ = ["UserSchemaCreateResponse", "UploadDetails", "UploadDetailsWarning"]


class UploadDetailsWarning(BaseModel):
    code: int
    """Code that identifies the event that occurred."""

    locations: Optional[List[str]] = None
    """JSONPath location(s) in the schema where these events were encountered.

    See
    [https://goessner.net/articles/JsonPath/](https://goessner.net/articles/JsonPath/)
    for JSONPath specification.
    """

    message: Optional[str] = None
    """Diagnostic message that describes the event."""


class UploadDetails(BaseModel):
    warnings: Optional[List[UploadDetailsWarning]] = None
    """Diagnostic warning events that occurred during processing.

    These events are non-critical errors found within the schema.
    """


class UserSchemaCreateResponse(BaseModel):
    schema_: OldPublicSchema = FieldInfo(alias="schema")

    upload_details: Optional[UploadDetails] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["DatasetCreateResponse"]


class DatasetCreateResponse(BaseModel):
    is_public: bool = FieldInfo(alias="isPublic")

    name: str

    uuid: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.workers.scripts import subdomain_create_params
from ....types.workers.scripts.subdomain_get_response import SubdomainGetResponse
from ....types.workers.scripts.subdomain_create_response import SubdomainCreateResponse
from ....types.workers.scripts.subdomain_delete_response import SubdomainDeleteResponse

__all__ = ["SubdomainResource", "AsyncSubdomainResource"]


class SubdomainResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SubdomainResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SubdomainResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SubdomainResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SubdomainResourceWithStreamingResponse(self)

    def create(
        self,
        script_name: str,
        *,
        account_id: str,
        enabled: bool,
        previews_enabled: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubdomainCreateResponse:
        """
        Enable or disable the Worker on the workers.dev subdomain.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          enabled: Whether the Worker should be available on the workers.dev subdomain.

          previews_enabled: Whether the Worker's Preview URLs should be available on the workers.dev
              subdomain.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/subdomain",
                account_id=account_id,
                script_name=script_name,
            ),
            body=maybe_transform(
                {
                    "enabled": enabled,
                    "previews_enabled": previews_enabled,
                },
                subdomain_create_params.SubdomainCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SubdomainCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[SubdomainCreateResponse], ResultWrapper[SubdomainCreateResponse]),
        )

    def delete(
        self,
        script_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubdomainDeleteResponse:
        """
        Disable all workers.dev subdomains for a Worker.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/subdomain",
                account_id=account_id,
                script_name=script_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SubdomainDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[SubdomainDeleteResponse], ResultWrapper[SubdomainDeleteResponse]),
        )

    def get(
        self,
        script_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubdomainGetResponse:
        """
        Get if the Worker is available on the workers.dev subdomain.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/subdomain",
                account_id=account_id,
                script_name=script_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SubdomainGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[SubdomainGetResponse], ResultWrapper[SubdomainGetResponse]),
        )


class AsyncSubdomainResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSubdomainResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSubdomainResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSubdomainResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSubdomainResourceWithStreamingResponse(self)

    async def create(
        self,
        script_name: str,
        *,
        account_id: str,
        enabled: bool,
        previews_enabled: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubdomainCreateResponse:
        """
        Enable or disable the Worker on the workers.dev subdomain.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          enabled: Whether the Worker should be available on the workers.dev subdomain.

          previews_enabled: Whether the Worker's Preview URLs should be available on the workers.dev
              subdomain.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/subdomain",
                account_id=account_id,
                script_name=script_name,
            ),
            body=await async_maybe_transform(
                {
                    "enabled": enabled,
                    "previews_enabled": previews_enabled,
                },
                subdomain_create_params.SubdomainCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SubdomainCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[SubdomainCreateResponse], ResultWrapper[SubdomainCreateResponse]),
        )

    async def delete(
        self,
        script_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubdomainDeleteResponse:
        """
        Disable all workers.dev subdomains for a Worker.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/subdomain",
                account_id=account_id,
                script_name=script_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SubdomainDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[SubdomainDeleteResponse], ResultWrapper[SubdomainDeleteResponse]),
        )

    async def get(
        self,
        script_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubdomainGetResponse:
        """
        Get if the Worker is available on the workers.dev subdomain.

        Args:
          account_id: Identifier.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/subdomain",
                account_id=account_id,
                script_name=script_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SubdomainGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[SubdomainGetResponse], ResultWrapper[SubdomainGetResponse]),
        )


class SubdomainResourceWithRawResponse:
    def __init__(self, subdomain: SubdomainResource) -> None:
        self._subdomain = subdomain

        self.create = to_raw_response_wrapper(
            subdomain.create,
        )
        self.delete = to_raw_response_wrapper(
            subdomain.delete,
        )
        self.get = to_raw_response_wrapper(
            subdomain.get,
        )


class AsyncSubdomainResourceWithRawResponse:
    def __init__(self, subdomain: AsyncSubdomainResource) -> None:
        self._subdomain = subdomain

        self.create = async_to_raw_response_wrapper(
            subdomain.create,
        )
        self.delete = async_to_raw_response_wrapper(
            subdomain.delete,
        )
        self.get = async_to_raw_response_wrapper(
            subdomain.get,
        )


class SubdomainResourceWithStreamingResponse:
    def __init__(self, subdomain: SubdomainResource) -> None:
        self._subdomain = subdomain

        self.create = to_streamed_response_wrapper(
            subdomain.create,
        )
        self.delete = to_streamed_response_wrapper(
            subdomain.delete,
        )
        self.get = to_streamed_response_wrapper(
            subdomain.get,
        )


class AsyncSubdomainResourceWithStreamingResponse:
    def __init__(self, subdomain: AsyncSubdomainResource) -> None:
        self._subdomain = subdomain

        self.create = async_to_streamed_response_wrapper(
            subdomain.create,
        )
        self.delete = async_to_streamed_response_wrapper(
            subdomain.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            subdomain.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["ExtendedEmailMatching"]


class ExtendedEmailMatching(BaseModel):
    """Configures user email settings for firewall policies.

    When you enable this, the system standardizes email addresses in the identity portion of the rule to match extended email variants in firewall policies. When you disable this setting, the system matches email addresses exactly as you provide them. Enable this setting if your email uses `.` or `+` modifiers.
    """

    enabled: Optional[bool] = None
    """Specify whether to match all variants of user emails (with + or .

    modifiers) used as criteria in Firewall policies.
    """

    read_only: Optional[bool] = None
    """
    Indicate that this setting was shared via the Orgs API and read only for the
    current account.
    """

    source_account: Optional[str] = None
    """Indicate the account tag of the account that shared this setting."""

    version: Optional[int] = None
    """Indicate the version number of the setting."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["ASNBotnetThreatFeedResponse", "Ase", "Meta"]


class Ase(BaseModel):
    asn: int

    country: str

    name: str

    rank: int

    rank_change: Optional[int] = FieldInfo(alias="rankChange", default=None)


class Meta(BaseModel):
    date: str

    total: int

    compare_date: Optional[str] = FieldInfo(alias="compareDate", default=None)


class ASNBotnetThreatFeedResponse(BaseModel):
    ases: List[Ase]

    meta: Meta



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from ...fallback_domain_param import FallbackDomainParam

__all__ = ["FallbackDomainUpdateParams"]


class FallbackDomainUpdateParams(TypedDict, total=False):
    account_id: Required[str]

    domains: Required[Iterable[FallbackDomainParam]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, Optional, cast
from typing_extensions import Literal

import httpx

from .logs import (
    LogsResource,
    AsyncLogsResource,
    LogsResourceWithRawResponse,
    AsyncLogsResourceWithRawResponse,
    LogsResourceWithStreamingResponse,
    AsyncLogsResourceWithStreamingResponse,
)
from .urls import (
    URLsResource,
    AsyncURLsResource,
    URLsResourceWithRawResponse,
    AsyncURLsResourceWithRawResponse,
    URLsResourceWithStreamingResponse,
    AsyncURLsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from .datasets import (
    DatasetsResource,
    AsyncDatasetsResource,
    DatasetsResourceWithRawResponse,
    AsyncDatasetsResourceWithRawResponse,
    DatasetsResourceWithStreamingResponse,
    AsyncDatasetsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from .evaluations import (
    EvaluationsResource,
    AsyncEvaluationsResource,
    EvaluationsResourceWithRawResponse,
    AsyncEvaluationsResourceWithRawResponse,
    EvaluationsResourceWithStreamingResponse,
    AsyncEvaluationsResourceWithStreamingResponse,
)
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from .billing.billing import (
    BillingResource,
    AsyncBillingResource,
    BillingResourceWithRawResponse,
    AsyncBillingResourceWithRawResponse,
    BillingResourceWithStreamingResponse,
    AsyncBillingResourceWithStreamingResponse,
)
from .dynamic_routing import (
    DynamicRoutingResource,
    AsyncDynamicRoutingResource,
    DynamicRoutingResourceWithRawResponse,
    AsyncDynamicRoutingResourceWithRawResponse,
    DynamicRoutingResourceWithStreamingResponse,
    AsyncDynamicRoutingResourceWithStreamingResponse,
)
from .evaluation_types import (
    EvaluationTypesResource,
    AsyncEvaluationTypesResource,
    EvaluationTypesResourceWithRawResponse,
    AsyncEvaluationTypesResourceWithRawResponse,
    EvaluationTypesResourceWithStreamingResponse,
    AsyncEvaluationTypesResourceWithStreamingResponse,
)
from .provider_configs import (
    ProviderConfigsResource,
    AsyncProviderConfigsResource,
    ProviderConfigsResourceWithRawResponse,
    AsyncProviderConfigsResourceWithRawResponse,
    ProviderConfigsResourceWithStreamingResponse,
    AsyncProviderConfigsResourceWithStreamingResponse,
)
from ...types.ai_gateway import ai_gateway_list_params, ai_gateway_create_params, ai_gateway_update_params
from ...types.ai_gateway.ai_gateway_get_response import AIGatewayGetResponse
from ...types.ai_gateway.ai_gateway_list_response import AIGatewayListResponse
from ...types.ai_gateway.ai_gateway_create_response import AIGatewayCreateResponse
from ...types.ai_gateway.ai_gateway_delete_response import AIGatewayDeleteResponse
from ...types.ai_gateway.ai_gateway_update_response import AIGatewayUpdateResponse

__all__ = ["AIGatewayResource", "AsyncAIGatewayResource"]


class AIGatewayResource(SyncAPIResource):
    @cached_property
    def evaluation_types(self) -> EvaluationTypesResource:
        return EvaluationTypesResource(self._client)

    @cached_property
    def logs(self) -> LogsResource:
        return LogsResource(self._client)

    @cached_property
    def datasets(self) -> DatasetsResource:
        return DatasetsResource(self._client)

    @cached_property
    def evaluations(self) -> EvaluationsResource:
        return EvaluationsResource(self._client)

    @cached_property
    def dynamic_routing(self) -> DynamicRoutingResource:
        return DynamicRoutingResource(self._client)

    @cached_property
    def provider_configs(self) -> ProviderConfigsResource:
        return ProviderConfigsResource(self._client)

    @cached_property
    def urls(self) -> URLsResource:
        return URLsResource(self._client)

    @cached_property
    def billing(self) -> BillingResource:
        return BillingResource(self._client)

    @cached_property
    def with_raw_response(self) -> AIGatewayResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AIGatewayResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AIGatewayResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AIGatewayResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        id: str,
        cache_invalidate_on_update: bool,
        cache_ttl: Optional[int],
        collect_logs: bool,
        rate_limiting_interval: Optional[int],
        rate_limiting_limit: Optional[int],
        authentication: bool | Omit = omit,
        log_management: Optional[int] | Omit = omit,
        log_management_strategy: Optional[Literal["STOP_INSERTING", "DELETE_OLDEST"]] | Omit = omit,
        logpush: bool | Omit = omit,
        logpush_public_key: Optional[str] | Omit = omit,
        rate_limiting_technique: Optional[Literal["fixed", "sliding"]] | Omit = omit,
        retry_backoff: Optional[Literal["constant", "linear", "exponential"]] | Omit = omit,
        retry_delay: Optional[int] | Omit = omit,
        retry_max_attempts: Optional[int] | Omit = omit,
        workers_ai_billing_mode: Literal["postpaid"] | Omit = omit,
        zdr: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AIGatewayCreateResponse:
        """
        Creates a new AI Gateway.

        Args:
          id: gateway id

          retry_backoff: Backoff strategy for retry delays

          retry_delay: Delay between retry attempts in milliseconds (0-5000)

          retry_max_attempts: Maximum number of retry attempts for failed requests (1-5)

          workers_ai_billing_mode: Controls how Workers AI inference calls routed through this gateway are billed.
              Only 'postpaid' is currently supported.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/ai-gateway/gateways", account_id=account_id),
            body=maybe_transform(
                {
                    "id": id,
                    "cache_invalidate_on_update": cache_invalidate_on_update,
                    "cache_ttl": cache_ttl,
                    "collect_logs": collect_logs,
                    "rate_limiting_interval": rate_limiting_interval,
                    "rate_limiting_limit": rate_limiting_limit,
                    "authentication": authentication,
                    "log_management": log_management,
                    "log_management_strategy": log_management_strategy,
                    "logpush": logpush,
                    "logpush_public_key": logpush_public_key,
                    "rate_limiting_technique": rate_limiting_technique,
                    "retry_backoff": retry_backoff,
                    "retry_delay": retry_delay,
                    "retry_max_attempts": retry_max_attempts,
                    "workers_ai_billing_mode": workers_ai_billing_mode,
                    "zdr": zdr,
                },
                ai_gateway_create_params.AIGatewayCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AIGatewayCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[AIGatewayCreateResponse], ResultWrapper[AIGatewayCreateResponse]),
        )

    def update(
        self,
        id: str,
        *,
        account_id: str,
        cache_invalidate_on_update: bool,
        cache_ttl: Optional[int],
        collect_logs: bool,
        rate_limiting_interval: Optional[int],
        rate_limiting_limit: Optional[int],
        authentication: bool | Omit = omit,
        dlp: ai_gateway_update_params.DLP | Omit = omit,
        guardrails: Optional[ai_gateway_update_params.Guardrails] | Omit = omit,
        log_management: Optional[int] | Omit = omit,
        log_management_strategy: Optional[Literal["STOP_INSERTING", "DELETE_OLDEST"]] | Omit = omit,
        logpush: bool | Omit = omit,
        logpush_public_key: Optional[str] | Omit = omit,
        otel: Optional[Iterable[ai_gateway_update_params.Otel]] | Omit = omit,
        rate_limiting_technique: Optional[Literal["fixed", "sliding"]] | Omit = omit,
        retry_backoff: Optional[Literal["constant", "linear", "exponential"]] | Omit = omit,
        retry_delay: Optional[int] | Omit = omit,
        retry_max_attempts: Optional[int] | Omit = omit,
        store_id: Optional[str] | Omit = omit,
        stripe: Optional[ai_gateway_update_params.Stripe] | Omit = omit,
        workers_ai_billing_mode: Literal["postpaid"] | Omit = omit,
        zdr: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AIGatewayUpdateResponse:
        """
        Updates an existing AI Gateway dataset.

        Args:
          id: gateway id

          retry_backoff: Backoff strategy for retry delays

          retry_delay: Delay between retry attempts in milliseconds (0-5000)

          retry_max_attempts: Maximum number of retry attempts for failed requests (1-5)

          workers_ai_billing_mode: Controls how Workers AI inference calls routed through this gateway are billed.
              Only 'postpaid' is currently supported.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            path_template("/accounts/{account_id}/ai-gateway/gateways/{id}", account_id=account_id, id=id),
            body=maybe_transform(
                {
                    "cache_invalidate_on_update": cache_invalidate_on_update,
                    "cache_ttl": cache_ttl,
                    "collect_logs": collect_logs,
                    "rate_limiting_interval": rate_limiting_interval,
                    "rate_limiting_limit": rate_limiting_limit,
                    "authentication": authentication,
                    "dlp": dlp,
                    "guardrails": guardrails,
                    "log_management": log_management,
                    "log_management_strategy": log_management_strategy,
                    "logpush": logpush,
                    "logpush_public_key": logpush_public_key,
                    "otel": otel,
                    "rate_limiting_technique": rate_limiting_technique,
                    "retry_backoff": retry_backoff,
                    "retry_delay": retry_delay,
                    "retry_max_attempts": retry_max_attempts,
                    "store_id": store_id,
                    "stripe": stripe,
                    "workers_ai_billing_mode": workers_ai_billing_mode,
                    "zdr": zdr,
                },
                ai_gateway_update_params.AIGatewayUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AIGatewayUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[AIGatewayUpdateResponse], ResultWrapper[AIGatewayUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[AIGatewayListResponse]:
        """
        Lists all AI Gateway evaluator types configured for the account.

        Args:
          search: Search by id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/ai-gateway/gateways", account_id=account_id),
            page=SyncV4PagePaginationArray[AIGatewayListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "search": search,
                    },
                    ai_gateway_list_params.AIGatewayListParams,
                ),
            ),
            model=AIGatewayListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AIGatewayDeleteResponse:
        """
        Deletes an AI Gateway dataset.

        Args:
          id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            path_template("/accounts/{account_id}/ai-gateway/gateways/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AIGatewayDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[AIGatewayDeleteResponse], ResultWrapper[AIGatewayDeleteResponse]),
        )

    def get(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AIGatewayGetResponse:
        """
        Retrieves details for a specific AI Gateway dataset.

        Args:
          id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template("/accounts/{account_id}/ai-gateway/gateways/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AIGatewayGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[AIGatewayGetResponse], ResultWrapper[AIGatewayGetResponse]),
        )


class AsyncAIGatewayResource(AsyncAPIResource):
    @cached_property
    def evaluation_types(self) -> AsyncEvaluationTypesResource:
        return AsyncEvaluationTypesResource(self._client)

    @cached_property
    def logs(self) -> AsyncLogsResource:
        return AsyncLogsResource(self._client)

    @cached_property
    def datasets(self) -> AsyncDatasetsResource:
        return AsyncDatasetsResource(self._client)

    @cached_property
    def evaluations(self) -> AsyncEvaluationsResource:
        return AsyncEvaluationsResource(self._client)

    @cached_property
    def dynamic_routing(self) -> AsyncDynamicRoutingResource:
        return AsyncDynamicRoutingResource(self._client)

    @cached_property
    def provider_configs(self) -> AsyncProviderConfigsResource:
        return AsyncProviderConfigsResource(self._client)

    @cached_property
    def urls(self) -> AsyncURLsResource:
        return AsyncURLsResource(self._client)

    @cached_property
    def billing(self) -> AsyncBillingResource:
        return AsyncBillingResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAIGatewayResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAIGatewayResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAIGatewayResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAIGatewayResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        id: str,
        cache_invalidate_on_update: bool,
        cache_ttl: Optional[int],
        collect_logs: bool,
        rate_limiting_interval: Optional[int],
        rate_limiting_limit: Optional[int],
        authentication: bool | Omit = omit,
        log_management: Optional[int] | Omit = omit,
        log_management_strategy: Optional[Literal["STOP_INSERTING", "DELETE_OLDEST"]] | Omit = omit,
        logpush: bool | Omit = omit,
        logpush_public_key: Optional[str] | Omit = omit,
        rate_limiting_technique: Optional[Literal["fixed", "sliding"]] | Omit = omit,
        retry_backoff: Optional[Literal["constant", "linear", "exponential"]] | Omit = omit,
        retry_delay: Optional[int] | Omit = omit,
        retry_max_attempts: Optional[int] | Omit = omit,
        workers_ai_billing_mode: Literal["postpaid"] | Omit = omit,
        zdr: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AIGatewayCreateResponse:
        """
        Creates a new AI Gateway.

        Args:
          id: gateway id

          retry_backoff: Backoff strategy for retry delays

          retry_delay: Delay between retry attempts in milliseconds (0-5000)

          retry_max_attempts: Maximum number of retry attempts for failed requests (1-5)

          workers_ai_billing_mode: Controls how Workers AI inference calls routed through this gateway are billed.
              Only 'postpaid' is currently supported.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/ai-gateway/gateways", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "id": id,
                    "cache_invalidate_on_update": cache_invalidate_on_update,
                    "cache_ttl": cache_ttl,
                    "collect_logs": collect_logs,
                    "rate_limiting_interval": rate_limiting_interval,
                    "rate_limiting_limit": rate_limiting_limit,
                    "authentication": authentication,
                    "log_management": log_management,
                    "log_management_strategy": log_management_strategy,
                    "logpush": logpush,
                    "logpush_public_key": logpush_public_key,
                    "rate_limiting_technique": rate_limiting_technique,
                    "retry_backoff": retry_backoff,
                    "retry_delay": retry_delay,
                    "retry_max_attempts": retry_max_attempts,
                    "workers_ai_billing_mode": workers_ai_billing_mode,
                    "zdr": zdr,
                },
                ai_gateway_create_params.AIGatewayCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AIGatewayCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[AIGatewayCreateResponse], ResultWrapper[AIGatewayCreateResponse]),
        )

    async def update(
        self,
        id: str,
        *,
        account_id: str,
        cache_invalidate_on_update: bool,
        cache_ttl: Optional[int],
        collect_logs: bool,
        rate_limiting_interval: Optional[int],
        rate_limiting_limit: Optional[int],
        authentication: bool | Omit = omit,
        dlp: ai_gateway_update_params.DLP | Omit = omit,
        guardrails: Optional[ai_gateway_update_params.Guardrails] | Omit = omit,
        log_management: Optional[int] | Omit = omit,
        log_management_strategy: Optional[Literal["STOP_INSERTING", "DELETE_OLDEST"]] | Omit = omit,
        logpush: bool | Omit = omit,
        logpush_public_key: Optional[str] | Omit = omit,
        otel: Optional[Iterable[ai_gateway_update_params.Otel]] | Omit = omit,
        rate_limiting_technique: Optional[Literal["fixed", "sliding"]] | Omit = omit,
        retry_backoff: Optional[Literal["constant", "linear", "exponential"]] | Omit = omit,
        retry_delay: Optional[int] | Omit = omit,
        retry_max_attempts: Optional[int] | Omit = omit,
        store_id: Optional[str] | Omit = omit,
        stripe: Optional[ai_gateway_update_params.Stripe] | Omit = omit,
        workers_ai_billing_mode: Literal["postpaid"] | Omit = omit,
        zdr: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AIGatewayUpdateResponse:
        """
        Updates an existing AI Gateway dataset.

        Args:
          id: gateway id

          retry_backoff: Backoff strategy for retry delays

          retry_delay: Delay between retry attempts in milliseconds (0-5000)

          retry_max_attempts: Maximum number of retry attempts for failed requests (1-5)

          workers_ai_billing_mode: Controls how Workers AI inference calls routed through this gateway are billed.
              Only 'postpaid' is currently supported.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/ai-gateway/gateways/{id}", account_id=account_id, id=id),
            body=await async_maybe_transform(
                {
                    "cache_invalidate_on_update": cache_invalidate_on_update,
                    "cache_ttl": cache_ttl,
                    "collect_logs": collect_logs,
                    "rate_limiting_interval": rate_limiting_interval,
                    "rate_limiting_limit": rate_limiting_limit,
                    "authentication": authentication,
                    "dlp": dlp,
                    "guardrails": guardrails,
                    "log_management": log_management,
                    "log_management_strategy": log_management_strategy,
                    "logpush": logpush,
                    "logpush_public_key": logpush_public_key,
                    "otel": otel,
                    "rate_limiting_technique": rate_limiting_technique,
                    "retry_backoff": retry_backoff,
                    "retry_delay": retry_delay,
                    "retry_max_attempts": retry_max_attempts,
                    "store_id": store_id,
                    "stripe": stripe,
                    "workers_ai_billing_mode": workers_ai_billing_mode,
                    "zdr": zdr,
                },
                ai_gateway_update_params.AIGatewayUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AIGatewayUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[AIGatewayUpdateResponse], ResultWrapper[AIGatewayUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AIGatewayListResponse, AsyncV4PagePaginationArray[AIGatewayListResponse]]:
        """
        Lists all AI Gateway evaluator types configured for the account.

        Args:
          search: Search by id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/ai-gateway/gateways", account_id=account_id),
            page=AsyncV4PagePaginationArray[AIGatewayListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "search": search,
                    },
                    ai_gateway_list_params.AIGatewayListParams,
                ),
            ),
            model=AIGatewayListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AIGatewayDeleteResponse:
        """
        Deletes an AI Gateway dataset.

        Args:
          id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            path_template("/accounts/{account_id}/ai-gateway/gateways/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AIGatewayDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[AIGatewayDeleteResponse], ResultWrapper[AIGatewayDeleteResponse]),
        )

    async def get(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AIGatewayGetResponse:
        """
        Retrieves details for a specific AI Gateway dataset.

        Args:
          id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/ai-gateway/gateways/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AIGatewayGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[AIGatewayGetResponse], ResultWrapper[AIGatewayGetResponse]),
        )


class AIGatewayResourceWithRawResponse:
    def __init__(self, ai_gateway: AIGatewayResource) -> None:
        self._ai_gateway = ai_gateway

        self.create = to_raw_response_wrapper(
            ai_gateway.create,
        )
        self.update = to_raw_response_wrapper(
            ai_gateway.update,
        )
        self.list = to_raw_response_wrapper(
            ai_gateway.list,
        )
        self.delete = to_raw_response_wrapper(
            ai_gateway.delete,
        )
        self.get = to_raw_response_wrapper(
            ai_gateway.get,
        )

    @cached_property
    def evaluation_types(self) -> EvaluationTypesResourceWithRawResponse:
        return EvaluationTypesResourceWithRawResponse(self._ai_gateway.evaluation_types)

    @cached_property
    def logs(self) -> LogsResourceWithRawResponse:
        return LogsResourceWithRawResponse(self._ai_gateway.logs)

    @cached_property
    def datasets(self) -> DatasetsResourceWithRawResponse:
        return DatasetsResourceWithRawResponse(self._ai_gateway.datasets)

    @cached_property
    def evaluations(self) -> EvaluationsResourceWithRawResponse:
        return EvaluationsResourceWithRawResponse(self._ai_gateway.evaluations)

    @cached_property
    def dynamic_routing(self) -> DynamicRoutingResourceWithRawResponse:
        return DynamicRoutingResourceWithRawResponse(self._ai_gateway.dynamic_routing)

    @cached_property
    def provider_configs(self) -> ProviderConfigsResourceWithRawResponse:
        return ProviderConfigsResourceWithRawResponse(self._ai_gateway.provider_configs)

    @cached_property
    def urls(self) -> URLsResourceWithRawResponse:
        return URLsResourceWithRawResponse(self._ai_gateway.urls)

    @cached_property
    def billing(self) -> BillingResourceWithRawResponse:
        return BillingResourceWithRawResponse(self._ai_gateway.billing)


class AsyncAIGatewayResourceWithRawResponse:
    def __init__(self, ai_gateway: AsyncAIGatewayResource) -> None:
        self._ai_gateway = ai_gateway

        self.create = async_to_raw_response_wrapper(
            ai_gateway.create,
        )
        self.update = async_to_raw_response_wrapper(
            ai_gateway.update,
        )
        self.list = async_to_raw_response_wrapper(
            ai_gateway.list,
        )
        self.delete = async_to_raw_response_wrapper(
            ai_gateway.delete,
        )
        self.get = async_to_raw_response_wrapper(
            ai_gateway.get,
        )

    @cached_property
    def evaluation_types(self) -> AsyncEvaluationTypesResourceWithRawResponse:
        return AsyncEvaluationTypesResourceWithRawResponse(self._ai_gateway.evaluation_types)

    @cached_property
    def logs(self) -> AsyncLogsResourceWithRawResponse:
        return AsyncLogsResourceWithRawResponse(self._ai_gateway.logs)

    @cached_property
    def datasets(self) -> AsyncDatasetsResourceWithRawResponse:
        return AsyncDatasetsResourceWithRawResponse(self._ai_gateway.datasets)

    @cached_property
    def evaluations(self) -> AsyncEvaluationsResourceWithRawResponse:
        return AsyncEvaluationsResourceWithRawResponse(self._ai_gateway.evaluations)

    @cached_property
    def dynamic_routing(self) -> AsyncDynamicRoutingResourceWithRawResponse:
        return AsyncDynamicRoutingResourceWithRawResponse(self._ai_gateway.dynamic_routing)

    @cached_property
    def provider_configs(self) -> AsyncProviderConfigsResourceWithRawResponse:
        return AsyncProviderConfigsResourceWithRawResponse(self._ai_gateway.provider_configs)

    @cached_property
    def urls(self) -> AsyncURLsResourceWithRawResponse:
        return AsyncURLsResourceWithRawResponse(self._ai_gateway.urls)

    @cached_property
    def billing(self) -> AsyncBillingResourceWithRawResponse:
        return AsyncBillingResourceWithRawResponse(self._ai_gateway.billing)


class AIGatewayResourceWithStreamingResponse:
    def __init__(self, ai_gateway: AIGatewayResource) -> None:
        self._ai_gateway = ai_gateway

        self.create = to_streamed_response_wrapper(
            ai_gateway.create,
        )
        self.update = to_streamed_response_wrapper(
            ai_gateway.update,
        )
        self.list = to_streamed_response_wrapper(
            ai_gateway.list,
        )
        self.delete = to_streamed_response_wrapper(
            ai_gateway.delete,
        )
        self.get = to_streamed_response_wrapper(
            ai_gateway.get,
        )

    @cached_property
    def evaluation_types(self) -> EvaluationTypesResourceWithStreamingResponse:
        return EvaluationTypesResourceWithStreamingResponse(self._ai_gateway.evaluation_types)

    @cached_property
    def logs(self) -> LogsResourceWithStreamingResponse:
        return LogsResourceWithStreamingResponse(self._ai_gateway.logs)

    @cached_property
    def datasets(self) -> DatasetsResourceWithStreamingResponse:
        return DatasetsResourceWithStreamingResponse(self._ai_gateway.datasets)

    @cached_property
    def evaluations(self) -> EvaluationsResourceWithStreamingResponse:
        return EvaluationsResourceWithStreamingResponse(self._ai_gateway.evaluations)

    @cached_property
    def dynamic_routing(self) -> DynamicRoutingResourceWithStreamingResponse:
        return DynamicRoutingResourceWithStreamingResponse(self._ai_gateway.dynamic_routing)

    @cached_property
    def provider_configs(self) -> ProviderConfigsResourceWithStreamingResponse:
        return ProviderConfigsResourceWithStreamingResponse(self._ai_gateway.provider_configs)

    @cached_property
    def urls(self) -> URLsResourceWithStreamingResponse:
        return URLsResourceWithStreamingResponse(self._ai_gateway.urls)

    @cached_property
    def billing(self) -> BillingResourceWithStreamingResponse:
        return BillingResourceWithStreamingResponse(self._ai_gateway.billing)


class AsyncAIGatewayResourceWithStreamingResponse:
    def __init__(self, ai_gateway: AsyncAIGatewayResource) -> None:
        self._ai_gateway = ai_gateway

        self.create = async_to_streamed_response_wrapper(
            ai_gateway.create,
        )
        self.update = async_to_streamed_response_wrapper(
            ai_gateway.update,
        )
        self.list = async_to_streamed_response_wrapper(
            ai_gateway.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            ai_gateway.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            ai_gateway.get,
        )

    @cached_property
    def evaluation_types(self) -> AsyncEvaluationTypesResourceWithStreamingResponse:
        return AsyncEvaluationTypesResourceWithStreamingResponse(self._ai_gateway.evaluation_types)

    @cached_property
    def logs(self) -> AsyncLogsResourceWithStreamingResponse:
        return AsyncLogsResourceWithStreamingResponse(self._ai_gateway.logs)

    @cached_property
    def datasets(self) -> AsyncDatasetsResourceWithStreamingResponse:
        return AsyncDatasetsResourceWithStreamingResponse(self._ai_gateway.datasets)

    @cached_property
    def evaluations(self) -> AsyncEvaluationsResourceWithStreamingResponse:
        return AsyncEvaluationsResourceWithStreamingResponse(self._ai_gateway.evaluations)

    @cached_property
    def dynamic_routing(self) -> AsyncDynamicRoutingResourceWithStreamingResponse:
        return AsyncDynamicRoutingResourceWithStreamingResponse(self._ai_gateway.dynamic_routing)

    @cached_property
    def provider_configs(self) -> AsyncProviderConfigsResourceWithStreamingResponse:
        return AsyncProviderConfigsResourceWithStreamingResponse(self._ai_gateway.provider_configs)

    @cached_property
    def urls(self) -> AsyncURLsResourceWithStreamingResponse:
        return AsyncURLsResourceWithStreamingResponse(self._ai_gateway.urls)

    @cached_property
    def billing(self) -> AsyncBillingResourceWithStreamingResponse:
        return AsyncBillingResourceWithStreamingResponse(self._ai_gateway.billing)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .settings import (
    SettingsResource,
    AsyncSettingsResource,
    SettingsResourceWithRawResponse,
    AsyncSettingsResourceWithRawResponse,
    SettingsResourceWithStreamingResponse,
    AsyncSettingsResourceWithStreamingResponse,
)
from .hostnames import (
    HostnamesResource,
    AsyncHostnamesResource,
    HostnamesResourceWithRawResponse,
    AsyncHostnamesResourceWithRawResponse,
    HostnamesResourceWithStreamingResponse,
    AsyncHostnamesResourceWithStreamingResponse,
)
from .zone_certificates import (
    ZoneCertificatesResource,
    AsyncZoneCertificatesResource,
    ZoneCertificatesResourceWithRawResponse,
    AsyncZoneCertificatesResourceWithRawResponse,
    ZoneCertificatesResourceWithStreamingResponse,
    AsyncZoneCertificatesResourceWithStreamingResponse,
)
from .hostname_certificates import (
    HostnameCertificatesResource,
    AsyncHostnameCertificatesResource,
    HostnameCertificatesResourceWithRawResponse,
    AsyncHostnameCertificatesResourceWithRawResponse,
    HostnameCertificatesResourceWithStreamingResponse,
    AsyncHostnameCertificatesResourceWithStreamingResponse,
)
from .origin_tls_client_auth import (
    OriginTLSClientAuthResource,
    AsyncOriginTLSClientAuthResource,
    OriginTLSClientAuthResourceWithRawResponse,
    AsyncOriginTLSClientAuthResourceWithRawResponse,
    OriginTLSClientAuthResourceWithStreamingResponse,
    AsyncOriginTLSClientAuthResourceWithStreamingResponse,
)

__all__ = [
    "ZoneCertificatesResource",
    "AsyncZoneCertificatesResource",
    "ZoneCertificatesResourceWithRawResponse",
    "AsyncZoneCertificatesResourceWithRawResponse",
    "ZoneCertificatesResourceWithStreamingResponse",
    "AsyncZoneCertificatesResourceWithStreamingResponse",
    "HostnamesResource",
    "AsyncHostnamesResource",
    "HostnamesResourceWithRawResponse",
    "AsyncHostnamesResourceWithRawResponse",
    "HostnamesResourceWithStreamingResponse",
    "AsyncHostnamesResourceWithStreamingResponse",
    "HostnameCertificatesResource",
    "AsyncHostnameCertificatesResource",
    "HostnameCertificatesResourceWithRawResponse",
    "AsyncHostnameCertificatesResourceWithRawResponse",
    "HostnameCertificatesResourceWithStreamingResponse",
    "AsyncHostnameCertificatesResourceWithStreamingResponse",
    "SettingsResource",
    "AsyncSettingsResource",
    "SettingsResourceWithRawResponse",
    "AsyncSettingsResourceWithRawResponse",
    "SettingsResourceWithStreamingResponse",
    "AsyncSettingsResourceWithStreamingResponse",
    "OriginTLSClientAuthResource",
    "AsyncOriginTLSClientAuthResource",
    "OriginTLSClientAuthResourceWithRawResponse",
    "AsyncOriginTLSClientAuthResourceWithRawResponse",
    "OriginTLSClientAuthResourceWithStreamingResponse",
    "AsyncOriginTLSClientAuthResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from .pages.test import Test
from .labeled_region import LabeledRegion

__all__ = ["PageListResponse"]


class PageListResponse(BaseModel):
    region: Optional[LabeledRegion] = None
    """A test region with a label."""

    schedule_frequency: Optional[Literal["DAILY", "WEEKLY"]] = FieldInfo(alias="scheduleFrequency", default=None)
    """The frequency of the test."""

    tests: Optional[List[Test]] = None

    url: Optional[str] = None
    """A URL."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["AccessRuleIPConfigurationParam"]


class AccessRuleIPConfigurationParam(TypedDict, total=False):
    target: Literal["ip"]
    """The configuration target.

    You must set the target to `ip` when specifying an IP address in the rule.
    """

    value: str
    """The IP address to match.

    This address will be compared to the IP address of incoming requests.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .filters import (
    FiltersResource,
    AsyncFiltersResource,
    FiltersResourceWithRawResponse,
    AsyncFiltersResourceWithRawResponse,
    FiltersResourceWithStreamingResponse,
    AsyncFiltersResourceWithStreamingResponse,
)

__all__ = [
    "FiltersResource",
    "AsyncFiltersResource",
    "FiltersResourceWithRawResponse",
    "AsyncFiltersResourceWithRawResponse",
    "FiltersResourceWithStreamingResponse",
    "AsyncFiltersResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["RequestListParams"]


class RequestListParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    page: Required[int]
    """Page number of results."""

    per_page: Required[int]
    """Number of results per page."""

    completed_after: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Retrieve requests completed after this time."""

    completed_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Retrieve requests completed before this time."""

    created_after: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Retrieve requests created after this time."""

    created_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Retrieve requests created before this time."""

    request_type: str
    """Requested information from request."""

    sort_by: str
    """Field to sort results by."""

    sort_order: Literal["asc", "desc"]
    """Sort order (asc or desc)."""

    status: Literal["open", "accepted", "reported", "approved", "completed", "declined"]
    """Request Status."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["RadarEmailSeries"]


class RadarEmailSeries(BaseModel):
    fail: List[str] = FieldInfo(alias="FAIL")

    none: List[str] = FieldInfo(alias="NONE")

    pass_: List[str] = FieldInfo(alias="PASS")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .....pagination import SyncV4PagePagination, AsyncV4PagePagination
from ....._base_client import AsyncPaginator, make_request_options
from .....types.radar.bgp.leaks import event_list_params
from .....types.radar.bgp.leaks.event_list_response import EventListResponse

__all__ = ["EventsResource", "AsyncEventsResource"]


class EventsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return EventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return EventsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        date_end: Union[str, datetime] | Omit = omit,
        date_range: str | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        event_id: int | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        involved_asn: int | Omit = omit,
        involved_country: str | Omit = omit,
        leak_asn: int | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        sort_by: Literal["ID", "LEAKS", "PEERS", "PREFIXES", "ORIGINS", "TIME"] | Omit = omit,
        sort_order: Literal["ASC", "DESC"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePagination[EventListResponse]:
        """
        Retrieves the BGP route leak events.

        Args:
          date_end: End of the date range (inclusive).

          date_range: Filters results by date range.

          date_start: Start of the date range (inclusive).

          event_id: The unique identifier of a event.

          format: Format in which results will be returned.

          involved_asn: ASN that is causing or affected by a route leak event.

          involved_country: Country code of a involved ASN in a route leak event.

          leak_asn: The leaking AS of a route leak event.

          page: Current page number, starting from 1.

          per_page: Number of entries per page.

          sort_by: Sorts results by the specified field.

          sort_order: Sort order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/radar/bgp/leaks/events",
            page=SyncV4PagePagination[EventListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "event_id": event_id,
                        "format": format,
                        "involved_asn": involved_asn,
                        "involved_country": involved_country,
                        "leak_asn": leak_asn,
                        "page": page,
                        "per_page": per_page,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                    },
                    event_list_params.EventListParams,
                ),
            ),
            model=EventListResponse,
        )


class AsyncEventsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncEventsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        date_end: Union[str, datetime] | Omit = omit,
        date_range: str | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        event_id: int | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        involved_asn: int | Omit = omit,
        involved_country: str | Omit = omit,
        leak_asn: int | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        sort_by: Literal["ID", "LEAKS", "PEERS", "PREFIXES", "ORIGINS", "TIME"] | Omit = omit,
        sort_order: Literal["ASC", "DESC"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[EventListResponse, AsyncV4PagePagination[EventListResponse]]:
        """
        Retrieves the BGP route leak events.

        Args:
          date_end: End of the date range (inclusive).

          date_range: Filters results by date range.

          date_start: Start of the date range (inclusive).

          event_id: The unique identifier of a event.

          format: Format in which results will be returned.

          involved_asn: ASN that is causing or affected by a route leak event.

          involved_country: Country code of a involved ASN in a route leak event.

          leak_asn: The leaking AS of a route leak event.

          page: Current page number, starting from 1.

          per_page: Number of entries per page.

          sort_by: Sorts results by the specified field.

          sort_order: Sort order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/radar/bgp/leaks/events",
            page=AsyncV4PagePagination[EventListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "event_id": event_id,
                        "format": format,
                        "involved_asn": involved_asn,
                        "involved_country": involved_country,
                        "leak_asn": leak_asn,
                        "page": page,
                        "per_page": per_page,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                    },
                    event_list_params.EventListParams,
                ),
            ),
            model=EventListResponse,
        )


class EventsResourceWithRawResponse:
    def __init__(self, events: EventsResource) -> None:
        self._events = events

        self.list = to_raw_response_wrapper(
            events.list,
        )


class AsyncEventsResourceWithRawResponse:
    def __init__(self, events: AsyncEventsResource) -> None:
        self._events = events

        self.list = async_to_raw_response_wrapper(
            events.list,
        )


class EventsResourceWithStreamingResponse:
    def __init__(self, events: EventsResource) -> None:
        self._events = events

        self.list = to_streamed_response_wrapper(
            events.list,
        )


class AsyncEventsResourceWithStreamingResponse:
    def __init__(self, events: AsyncEventsResource) -> None:
        self._events = events

        self.list = async_to_streamed_response_wrapper(
            events.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo
from ..buckets.provider import Provider

__all__ = [
    "ConnectivityPrecheckSourceParams",
    "R2SlurperS3SourceSchema",
    "R2SlurperS3SourceSchemaSecret",
    "R2SlurperGcsSourceSchema",
    "R2SlurperGcsSourceSchemaSecret",
    "R2SlurperR2SourceSchema",
    "R2SlurperR2SourceSchemaSecret",
]


class R2SlurperS3SourceSchema(TypedDict, total=False):
    account_id: Required[str]

    bucket: Required[str]

    secret: Required[R2SlurperS3SourceSchemaSecret]

    vendor: Required[Literal["s3"]]

    endpoint: Optional[str]

    keys: Optional[SequenceNotStr[str]]

    path_prefix: Annotated[Optional[str], PropertyInfo(alias="pathPrefix")]

    region: Optional[str]


class R2SlurperS3SourceSchemaSecret(TypedDict, total=False):
    access_key_id: Required[Annotated[str, PropertyInfo(alias="accessKeyId")]]

    secret_access_key: Required[Annotated[str, PropertyInfo(alias="secretAccessKey")]]


class R2SlurperGcsSourceSchema(TypedDict, total=False):
    account_id: Required[str]

    bucket: Required[str]

    secret: Required[R2SlurperGcsSourceSchemaSecret]

    vendor: Required[Literal["gcs"]]

    keys: Optional[SequenceNotStr[str]]

    path_prefix: Annotated[Optional[str], PropertyInfo(alias="pathPrefix")]


class R2SlurperGcsSourceSchemaSecret(TypedDict, total=False):
    client_email: Required[Annotated[str, PropertyInfo(alias="clientEmail")]]

    private_key: Required[Annotated[str, PropertyInfo(alias="privateKey")]]


class R2SlurperR2SourceSchema(TypedDict, total=False):
    account_id: Required[str]

    bucket: Required[str]

    secret: Required[R2SlurperR2SourceSchemaSecret]

    vendor: Required[Provider]

    jurisdiction: Literal["default", "eu", "fedramp"]

    keys: Optional[SequenceNotStr[str]]

    path_prefix: Annotated[Optional[str], PropertyInfo(alias="pathPrefix")]


class R2SlurperR2SourceSchemaSecret(TypedDict, total=False):
    access_key_id: Required[Annotated[str, PropertyInfo(alias="accessKeyId")]]

    secret_access_key: Required[Annotated[str, PropertyInfo(alias="secretAccessKey")]]


ConnectivityPrecheckSourceParams: TypeAlias = Union[
    R2SlurperS3SourceSchema, R2SlurperGcsSourceSchema, R2SlurperR2SourceSchema
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["Group"]


class Group(BaseModel):
    id: str
    """Defines the unique identifier of the rule group."""

    description: Optional[str] = None
    """Defines an informative summary of what the rule group does."""

    mode: Literal["on", "off"]
    """Defines the state of the rules contained in the rule group.

    When `on`, the rules in the group are configurable/usable.
    """

    name: str
    """Defines the name of the rule group."""

    rules_count: float
    """Defines the number of rules in the current rule group."""

    allowed_modes: Optional[List[Literal["on", "off"]]] = None
    """Defines the available states for the rule group."""

    modified_rules_count: Optional[float] = None
    """
    Defines the number of rules within the group that have been modified from their
    default configuration.
    """

    package_id: Optional[str] = None
    """Defines the unique identifier of a WAF package."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, List, cast
from typing_extensions import Literal

import httpx

from .keys import (
    KeysResource,
    AsyncKeysResource,
    KeysResourceWithRawResponse,
    AsyncKeysResourceWithRawResponse,
    KeysResourceWithStreamingResponse,
    AsyncKeysResourceWithStreamingResponse,
)
from .values import (
    ValuesResource,
    AsyncValuesResource,
    ValuesResourceWithRawResponse,
    AsyncValuesResourceWithRawResponse,
    ValuesResourceWithStreamingResponse,
    AsyncValuesResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform
from ..._compat import cached_property
from .zone_tags import (
    ZoneTagsResource,
    AsyncZoneTagsResource,
    ZoneTagsResourceWithRawResponse,
    AsyncZoneTagsResourceWithRawResponse,
    ZoneTagsResourceWithStreamingResponse,
    AsyncZoneTagsResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursorPaginationAfter, AsyncCursorPaginationAfter
from .account_tags import (
    AccountTagsResource,
    AsyncAccountTagsResource,
    AccountTagsResourceWithRawResponse,
    AsyncAccountTagsResourceWithRawResponse,
    AccountTagsResourceWithStreamingResponse,
    AsyncAccountTagsResourceWithStreamingResponse,
)
from ..._base_client import AsyncPaginator, make_request_options
from ...types.resource_tagging import resource_tagging_list_params
from ...types.resource_tagging.resource_tagging_list_response import ResourceTaggingListResponse

__all__ = ["ResourceTaggingResource", "AsyncResourceTaggingResource"]


class ResourceTaggingResource(SyncAPIResource):
    @cached_property
    def account_tags(self) -> AccountTagsResource:
        return AccountTagsResource(self._client)

    @cached_property
    def zone_tags(self) -> ZoneTagsResource:
        return ZoneTagsResource(self._client)

    @cached_property
    def keys(self) -> KeysResource:
        return KeysResource(self._client)

    @cached_property
    def values(self) -> ValuesResource:
        return ValuesResource(self._client)

    @cached_property
    def with_raw_response(self) -> ResourceTaggingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ResourceTaggingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ResourceTaggingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ResourceTaggingResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        tag: SequenceNotStr[str] | Omit = omit,
        type: List[
            Literal[
                "access_application",
                "access_application_policy",
                "access_group",
                "account",
                "ai_gateway",
                "alerting_policy",
                "alerting_webhook",
                "api_gateway_operation",
                "cloudflared_tunnel",
                "custom_certificate",
                "custom_hostname",
                "d1_database",
                "dns_record",
                "durable_object_namespace",
                "gateway_list",
                "gateway_rule",
                "image",
                "kv_namespace",
                "managed_client_certificate",
                "queue",
                "r2_bucket",
                "resource_share",
                "stream_live_input",
                "stream_video",
                "worker",
                "worker_version",
                "zone",
            ]
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPaginationAfter[ResourceTaggingListResponse]:
        """
        Lists all tagged resources for an account.

        Args:
          account_id: Identifier.

          cursor: Cursor for pagination.

          tag: Filter resources by tag criteria. This parameter can be repeated multiple times,
              with AND logic between parameters.

              Supported syntax:

              - **Key-only**: `tag=<key>` - Resource must have the tag key (e.g.,
                `tag=production`)
              - **Key-value**: `tag=<key>=<value>` - Resource must have the tag with specific
                value (e.g., `tag=env=prod`)
              - **Multiple values (OR)**: `tag=<key>=<v1>,<v2>` - Resource must have tag with
                any of the values (e.g., `tag=env=prod,staging`)
              - **Negate key-only**: `tag=!<key>` - Resource must not have the tag key (e.g.,
                `tag=!archived`)
              - **Negate key-value**: `tag=<key>!=<value>` - Resource must not have the tag
                with specific value (e.g., `tag=region!=us-west-1`)

              Multiple tag parameters are combined with AND logic.

          type: Filter by resource type. Can be repeated to filter by multiple types (OR logic).
              Example: ?type=zone&type=worker

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/tags/resources", account_id=account_id),
            page=SyncCursorPaginationAfter[ResourceTaggingListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "tag": tag,
                        "type": type,
                    },
                    resource_tagging_list_params.ResourceTaggingListParams,
                ),
            ),
            model=cast(
                Any, ResourceTaggingListResponse
            ),  # Union types cannot be passed in as arguments in the type system
        )


class AsyncResourceTaggingResource(AsyncAPIResource):
    @cached_property
    def account_tags(self) -> AsyncAccountTagsResource:
        return AsyncAccountTagsResource(self._client)

    @cached_property
    def zone_tags(self) -> AsyncZoneTagsResource:
        return AsyncZoneTagsResource(self._client)

    @cached_property
    def keys(self) -> AsyncKeysResource:
        return AsyncKeysResource(self._client)

    @cached_property
    def values(self) -> AsyncValuesResource:
        return AsyncValuesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncResourceTaggingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncResourceTaggingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncResourceTaggingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncResourceTaggingResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        tag: SequenceNotStr[str] | Omit = omit,
        type: List[
            Literal[
                "access_application",
                "access_application_policy",
                "access_group",
                "account",
                "ai_gateway",
                "alerting_policy",
                "alerting_webhook",
                "api_gateway_operation",
                "cloudflared_tunnel",
                "custom_certificate",
                "custom_hostname",
                "d1_database",
                "dns_record",
                "durable_object_namespace",
                "gateway_list",
                "gateway_rule",
                "image",
                "kv_namespace",
                "managed_client_certificate",
                "queue",
                "r2_bucket",
                "resource_share",
                "stream_live_input",
                "stream_video",
                "worker",
                "worker_version",
                "zone",
            ]
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ResourceTaggingListResponse, AsyncCursorPaginationAfter[ResourceTaggingListResponse]]:
        """
        Lists all tagged resources for an account.

        Args:
          account_id: Identifier.

          cursor: Cursor for pagination.

          tag: Filter resources by tag criteria. This parameter can be repeated multiple times,
              with AND logic between parameters.

              Supported syntax:

              - **Key-only**: `tag=<key>` - Resource must have the tag key (e.g.,
                `tag=production`)
              - **Key-value**: `tag=<key>=<value>` - Resource must have the tag with specific
                value (e.g., `tag=env=prod`)
              - **Multiple values (OR)**: `tag=<key>=<v1>,<v2>` - Resource must have tag with
                any of the values (e.g., `tag=env=prod,staging`)
              - **Negate key-only**: `tag=!<key>` - Resource must not have the tag key (e.g.,
                `tag=!archived`)
              - **Negate key-value**: `tag=<key>!=<value>` - Resource must not have the tag
                with specific value (e.g., `tag=region!=us-west-1`)

              Multiple tag parameters are combined with AND logic.

          type: Filter by resource type. Can be repeated to filter by multiple types (OR logic).
              Example: ?type=zone&type=worker

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/tags/resources", account_id=account_id),
            page=AsyncCursorPaginationAfter[ResourceTaggingListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "tag": tag,
                        "type": type,
                    },
                    resource_tagging_list_params.ResourceTaggingListParams,
                ),
            ),
            model=cast(
                Any, ResourceTaggingListResponse
            ),  # Union types cannot be passed in as arguments in the type system
        )


class ResourceTaggingResourceWithRawResponse:
    def __init__(self, resource_tagging: ResourceTaggingResource) -> None:
        self._resource_tagging = resource_tagging

        self.list = to_raw_response_wrapper(
            resource_tagging.list,
        )

    @cached_property
    def account_tags(self) -> AccountTagsResourceWithRawResponse:
        return AccountTagsResourceWithRawResponse(self._resource_tagging.account_tags)

    @cached_property
    def zone_tags(self) -> ZoneTagsResourceWithRawResponse:
        return ZoneTagsResourceWithRawResponse(self._resource_tagging.zone_tags)

    @cached_property
    def keys(self) -> KeysResourceWithRawResponse:
        return KeysResourceWithRawResponse(self._resource_tagging.keys)

    @cached_property
    def values(self) -> ValuesResourceWithRawResponse:
        return ValuesResourceWithRawResponse(self._resource_tagging.values)


class AsyncResourceTaggingResourceWithRawResponse:
    def __init__(self, resource_tagging: AsyncResourceTaggingResource) -> None:
        self._resource_tagging = resource_tagging

        self.list = async_to_raw_response_wrapper(
            resource_tagging.list,
        )

    @cached_property
    def account_tags(self) -> AsyncAccountTagsResourceWithRawResponse:
        return AsyncAccountTagsResourceWithRawResponse(self._resource_tagging.account_tags)

    @cached_property
    def zone_tags(self) -> AsyncZoneTagsResourceWithRawResponse:
        return AsyncZoneTagsResourceWithRawResponse(self._resource_tagging.zone_tags)

    @cached_property
    def keys(self) -> AsyncKeysResourceWithRawResponse:
        return AsyncKeysResourceWithRawResponse(self._resource_tagging.keys)

    @cached_property
    def values(self) -> AsyncValuesResourceWithRawResponse:
        return AsyncValuesResourceWithRawResponse(self._resource_tagging.values)


class ResourceTaggingResourceWithStreamingResponse:
    def __init__(self, resource_tagging: ResourceTaggingResource) -> None:
        self._resource_tagging = resource_tagging

        self.list = to_streamed_response_wrapper(
            resource_tagging.list,
        )

    @cached_property
    def account_tags(self) -> AccountTagsResourceWithStreamingResponse:
        return AccountTagsResourceWithStreamingResponse(self._resource_tagging.account_tags)

    @cached_property
    def zone_tags(self) -> ZoneTagsResourceWithStreamingResponse:
        return ZoneTagsResourceWithStreamingResponse(self._resource_tagging.zone_tags)

    @cached_property
    def keys(self) -> KeysResourceWithStreamingResponse:
        return KeysResourceWithStreamingResponse(self._resource_tagging.keys)

    @cached_property
    def values(self) -> ValuesResourceWithStreamingResponse:
        return ValuesResourceWithStreamingResponse(self._resource_tagging.values)


class AsyncResourceTaggingResourceWithStreamingResponse:
    def __init__(self, resource_tagging: AsyncResourceTaggingResource) -> None:
        self._resource_tagging = resource_tagging

        self.list = async_to_streamed_response_wrapper(
            resource_tagging.list,
        )

    @cached_property
    def account_tags(self) -> AsyncAccountTagsResourceWithStreamingResponse:
        return AsyncAccountTagsResourceWithStreamingResponse(self._resource_tagging.account_tags)

    @cached_property
    def zone_tags(self) -> AsyncZoneTagsResourceWithStreamingResponse:
        return AsyncZoneTagsResourceWithStreamingResponse(self._resource_tagging.zone_tags)

    @cached_property
    def keys(self) -> AsyncKeysResourceWithStreamingResponse:
        return AsyncKeysResourceWithStreamingResponse(self._resource_tagging.keys)

    @cached_property
    def values(self) -> AsyncValuesResourceWithStreamingResponse:
        return AsyncValuesResourceWithStreamingResponse(self._resource_tagging.values)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional

from .waf_rule import WAFRule
from ...._models import BaseModel
from .override_url import OverrideURL
from .rewrite_action import RewriteAction

__all__ = ["Override"]


class Override(BaseModel):
    id: Optional[str] = None
    """The unique identifier of the WAF override."""

    description: Optional[str] = None
    """An informative summary of the current URI-based WAF override."""

    groups: Optional[Dict[str, object]] = None
    """
    An object that allows you to enable or disable WAF rule groups for the current
    WAF override. Each key of this object must be the ID of a WAF rule group, and
    each value must be a valid WAF action (usually `default` or `disable`). When
    creating a new URI-based WAF override, you must provide a `groups` object or a
    `rules` object.
    """

    paused: Optional[bool] = None
    """When true, indicates that the rule is currently paused."""

    priority: Optional[float] = None
    """
    The relative priority of the current URI-based WAF override when multiple
    overrides match a single URL. A lower number indicates higher priority. Higher
    priority overrides may overwrite values set by lower priority overrides.
    """

    rewrite_action: Optional[RewriteAction] = None
    """
    Specifies that, when a WAF rule matches, its configured action will be replaced
    by the action configured in this object.
    """

    rules: Optional[WAFRule] = None
    """An object that allows you to override the action of specific WAF rules.

    Each key of this object must be the ID of a WAF rule, and each value must be a
    valid WAF action. Unless you are disabling a rule, ensure that you also enable
    the rule group that this WAF rule belongs to. When creating a new URI-based WAF
    override, you must provide a `groups` object or a `rules` object.
    """

    urls: Optional[List[OverrideURL]] = None
    """The URLs to include in the current WAF override.

    You can use wildcards. Each entered URL will be escaped before use, which means
    you can only use simple wildcard patterns.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .dispatch.dispatch import (
    DispatchResource,
    AsyncDispatchResource,
    DispatchResourceWithRawResponse,
    AsyncDispatchResourceWithRawResponse,
    DispatchResourceWithStreamingResponse,
    AsyncDispatchResourceWithStreamingResponse,
)

__all__ = ["WorkersForPlatformsResource", "AsyncWorkersForPlatformsResource"]


class WorkersForPlatformsResource(SyncAPIResource):
    @cached_property
    def dispatch(self) -> DispatchResource:
        return DispatchResource(self._client)

    @cached_property
    def with_raw_response(self) -> WorkersForPlatformsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return WorkersForPlatformsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WorkersForPlatformsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return WorkersForPlatformsResourceWithStreamingResponse(self)


class AsyncWorkersForPlatformsResource(AsyncAPIResource):
    @cached_property
    def dispatch(self) -> AsyncDispatchResource:
        return AsyncDispatchResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncWorkersForPlatformsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWorkersForPlatformsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWorkersForPlatformsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncWorkersForPlatformsResourceWithStreamingResponse(self)


class WorkersForPlatformsResourceWithRawResponse:
    def __init__(self, workers_for_platforms: WorkersForPlatformsResource) -> None:
        self._workers_for_platforms = workers_for_platforms

    @cached_property
    def dispatch(self) -> DispatchResourceWithRawResponse:
        return DispatchResourceWithRawResponse(self._workers_for_platforms.dispatch)


class AsyncWorkersForPlatformsResourceWithRawResponse:
    def __init__(self, workers_for_platforms: AsyncWorkersForPlatformsResource) -> None:
        self._workers_for_platforms = workers_for_platforms

    @cached_property
    def dispatch(self) -> AsyncDispatchResourceWithRawResponse:
        return AsyncDispatchResourceWithRawResponse(self._workers_for_platforms.dispatch)


class WorkersForPlatformsResourceWithStreamingResponse:
    def __init__(self, workers_for_platforms: WorkersForPlatformsResource) -> None:
        self._workers_for_platforms = workers_for_platforms

    @cached_property
    def dispatch(self) -> DispatchResourceWithStreamingResponse:
        return DispatchResourceWithStreamingResponse(self._workers_for_platforms.dispatch)


class AsyncWorkersForPlatformsResourceWithStreamingResponse:
    def __init__(self, workers_for_platforms: AsyncWorkersForPlatformsResource) -> None:
        self._workers_for_platforms = workers_for_platforms

    @cached_property
    def dispatch(self) -> AsyncDispatchResourceWithStreamingResponse:
        return AsyncDispatchResourceWithStreamingResponse(self._workers_for_platforms.dispatch)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.realtime_kit import (
    recording_get_recordings_params,
    recording_start_recordings_params,
    recording_start_track_recording_params,
    recording_pause_resume_stop_recording_params,
)
from ...types.realtime_kit.recording_get_recordings_response import RecordingGetRecordingsResponse
from ...types.realtime_kit.recording_start_recordings_response import RecordingStartRecordingsResponse
from ...types.realtime_kit.recording_get_one_recording_response import RecordingGetOneRecordingResponse
from ...types.realtime_kit.recording_get_active_recordings_response import RecordingGetActiveRecordingsResponse
from ...types.realtime_kit.recording_pause_resume_stop_recording_response import (
    RecordingPauseResumeStopRecordingResponse,
)

__all__ = ["RecordingsResource", "AsyncRecordingsResource"]


class RecordingsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RecordingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RecordingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RecordingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RecordingsResourceWithStreamingResponse(self)

    def get_active_recordings(
        self,
        meeting_id: str,
        *,
        account_id: str,
        app_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecordingGetActiveRecordingsResponse:
        """
        Returns the active recording details for the given meeting ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/recordings/active-recording/{meeting_id}",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecordingGetActiveRecordingsResponse,
        )

    def get_one_recording(
        self,
        recording_id: str,
        *,
        account_id: str,
        app_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecordingGetOneRecordingResponse:
        """
        Returns details of a recording for the given recording ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not recording_id:
            raise ValueError(f"Expected a non-empty value for `recording_id` but received {recording_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/recordings/{recording_id}",
                account_id=account_id,
                app_id=app_id,
                recording_id=recording_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecordingGetOneRecordingResponse,
        )

    def get_recordings(
        self,
        app_id: str,
        *,
        account_id: str,
        end_time: Union[str, datetime] | Omit = omit,
        expired: bool | Omit = omit,
        meeting_id: str | Omit = omit,
        page_no: float | Omit = omit,
        per_page: float | Omit = omit,
        search: str | Omit = omit,
        sort_by: Literal["invokedTime"] | Omit = omit,
        sort_order: Literal["ASC", "DESC"] | Omit = omit,
        start_time: Union[str, datetime] | Omit = omit,
        status: List[Literal["INVOKED", "RECORDING", "UPLOADING", "UPLOADED"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecordingGetRecordingsResponse:
        """Returns all recordings for an App.

        If the `meeting_id` parameter is passed,
        returns all recordings for the given meeting ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          end_time: The end time range for which you want to retrieve the meetings. The time must be
              specified in ISO format.

          expired: If passed, only shows expired/non-expired recordings on RealtimeKit's bucket

          meeting_id: ID of a meeting. Optional. Will limit results to only this meeting if passed.

          page_no: The page number from which you want your page search results to be displayed.

          per_page: Number of results per page

          search: The search query string. You can search using the meeting ID or title.

          start_time: The start time range for which you want to retrieve the meetings. The time must
              be specified in ISO format.

          status: Filter by one or more recording status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/recordings", account_id=account_id, app_id=app_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "end_time": end_time,
                        "expired": expired,
                        "meeting_id": meeting_id,
                        "page_no": page_no,
                        "per_page": per_page,
                        "search": search,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                        "start_time": start_time,
                        "status": status,
                    },
                    recording_get_recordings_params.RecordingGetRecordingsParams,
                ),
            ),
            cast_to=RecordingGetRecordingsResponse,
        )

    def pause_resume_stop_recording(
        self,
        recording_id: str,
        *,
        account_id: str,
        app_id: str,
        action: Literal["stop", "pause", "resume"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecordingPauseResumeStopRecordingResponse:
        """
        Pause/Resume/Stop a given recording ID.

        Args:
          account_id: The account identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not recording_id:
            raise ValueError(f"Expected a non-empty value for `recording_id` but received {recording_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/recordings/{recording_id}",
                account_id=account_id,
                app_id=app_id,
                recording_id=recording_id,
            ),
            body=maybe_transform(
                {"action": action}, recording_pause_resume_stop_recording_params.RecordingPauseResumeStopRecordingParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecordingPauseResumeStopRecordingResponse,
        )

    def start_recordings(
        self,
        app_id: str,
        *,
        account_id: str,
        allow_multiple_recordings: bool | Omit = omit,
        audio_config: recording_start_recordings_params.AudioConfig | Omit = omit,
        file_name_prefix: str | Omit = omit,
        interactive_config: recording_start_recordings_params.InteractiveConfig | Omit = omit,
        max_seconds: int | Omit = omit,
        meeting_id: str | Omit = omit,
        realtimekit_bucket_config: recording_start_recordings_params.RealtimekitBucketConfig | Omit = omit,
        rtmp_out_config: recording_start_recordings_params.RtmpOutConfig | Omit = omit,
        storage_config: Optional[recording_start_recordings_params.StorageConfig] | Omit = omit,
        url: str | Omit = omit,
        video_config: recording_start_recordings_params.VideoConfig | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecordingStartRecordingsResponse:
        """Starts recording a meeting.

        The meeting can be started by an App admin directly,
        or a participant with permissions to start a recording, based on the type of
        authorization used.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          allow_multiple_recordings: By default, a meeting allows only one recording to run at a time. Enabling the
              `allow_multiple_recordings` parameter to true allows you to initiate multiple
              recordings concurrently in the same meeting. This allows you to record separate
              videos of the same meeting with different configurations, such as portrait mode
              or landscape mode.

          audio_config: Object containing configuration regarding the audio that is being recorded.

          file_name_prefix: Update the recording file name.

          interactive_config: Allows you to add timed metadata to your recordings, which are digital markers
              inserted into a video file to provide contextual information at specific points
              in the content range. The ID3 tags containing this information are available to
              clients on the playback timeline in HLS format. The output files are generated
              in a compressed .tar format.

          max_seconds: Specifies the maximum duration for recording in seconds, ranging from a minimum
              of 60 seconds to a maximum of 24 hours.

          meeting_id: ID of the meeting to record.

          url: Pass a custom url to record arbitary screen

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/recordings", account_id=account_id, app_id=app_id
            ),
            body=maybe_transform(
                {
                    "allow_multiple_recordings": allow_multiple_recordings,
                    "audio_config": audio_config,
                    "file_name_prefix": file_name_prefix,
                    "interactive_config": interactive_config,
                    "max_seconds": max_seconds,
                    "meeting_id": meeting_id,
                    "realtimekit_bucket_config": realtimekit_bucket_config,
                    "rtmp_out_config": rtmp_out_config,
                    "storage_config": storage_config,
                    "url": url,
                    "video_config": video_config,
                },
                recording_start_recordings_params.RecordingStartRecordingsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecordingStartRecordingsResponse,
        )

    def start_track_recording(
        self,
        app_id: str,
        *,
        account_id: str,
        layers: Dict[str, recording_start_track_recording_params.Layers],
        meeting_id: str,
        max_seconds: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Starts a track recording in a meeting.

        Track recordings consist of "layers".
        Layers are used to map audio/video tracks in a meeting to output destinations.
        More information about track recordings is available in the
        [Track Recordings Guide Page](https://docs.realtime.cloudflare.com/guides/capabilities/recording/recording-overview).

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          meeting_id: ID of the meeting to record.

          max_seconds: Maximum seconds this recording should be active for (beta)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/recordings/track", account_id=account_id, app_id=app_id
            ),
            body=maybe_transform(
                {
                    "layers": layers,
                    "meeting_id": meeting_id,
                    "max_seconds": max_seconds,
                },
                recording_start_track_recording_params.RecordingStartTrackRecordingParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncRecordingsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRecordingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRecordingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRecordingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRecordingsResourceWithStreamingResponse(self)

    async def get_active_recordings(
        self,
        meeting_id: str,
        *,
        account_id: str,
        app_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecordingGetActiveRecordingsResponse:
        """
        Returns the active recording details for the given meeting ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/recordings/active-recording/{meeting_id}",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecordingGetActiveRecordingsResponse,
        )

    async def get_one_recording(
        self,
        recording_id: str,
        *,
        account_id: str,
        app_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecordingGetOneRecordingResponse:
        """
        Returns details of a recording for the given recording ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not recording_id:
            raise ValueError(f"Expected a non-empty value for `recording_id` but received {recording_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/recordings/{recording_id}",
                account_id=account_id,
                app_id=app_id,
                recording_id=recording_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecordingGetOneRecordingResponse,
        )

    async def get_recordings(
        self,
        app_id: str,
        *,
        account_id: str,
        end_time: Union[str, datetime] | Omit = omit,
        expired: bool | Omit = omit,
        meeting_id: str | Omit = omit,
        page_no: float | Omit = omit,
        per_page: float | Omit = omit,
        search: str | Omit = omit,
        sort_by: Literal["invokedTime"] | Omit = omit,
        sort_order: Literal["ASC", "DESC"] | Omit = omit,
        start_time: Union[str, datetime] | Omit = omit,
        status: List[Literal["INVOKED", "RECORDING", "UPLOADING", "UPLOADED"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecordingGetRecordingsResponse:
        """Returns all recordings for an App.

        If the `meeting_id` parameter is passed,
        returns all recordings for the given meeting ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          end_time: The end time range for which you want to retrieve the meetings. The time must be
              specified in ISO format.

          expired: If passed, only shows expired/non-expired recordings on RealtimeKit's bucket

          meeting_id: ID of a meeting. Optional. Will limit results to only this meeting if passed.

          page_no: The page number from which you want your page search results to be displayed.

          per_page: Number of results per page

          search: The search query string. You can search using the meeting ID or title.

          start_time: The start time range for which you want to retrieve the meetings. The time must
              be specified in ISO format.

          status: Filter by one or more recording status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/recordings", account_id=account_id, app_id=app_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "end_time": end_time,
                        "expired": expired,
                        "meeting_id": meeting_id,
                        "page_no": page_no,
                        "per_page": per_page,
                        "search": search,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                        "start_time": start_time,
                        "status": status,
                    },
                    recording_get_recordings_params.RecordingGetRecordingsParams,
                ),
            ),
            cast_to=RecordingGetRecordingsResponse,
        )

    async def pause_resume_stop_recording(
        self,
        recording_id: str,
        *,
        account_id: str,
        app_id: str,
        action: Literal["stop", "pause", "resume"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecordingPauseResumeStopRecordingResponse:
        """
        Pause/Resume/Stop a given recording ID.

        Args:
          account_id: The account identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not recording_id:
            raise ValueError(f"Expected a non-empty value for `recording_id` but received {recording_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/recordings/{recording_id}",
                account_id=account_id,
                app_id=app_id,
                recording_id=recording_id,
            ),
            body=await async_maybe_transform(
                {"action": action}, recording_pause_resume_stop_recording_params.RecordingPauseResumeStopRecordingParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecordingPauseResumeStopRecordingResponse,
        )

    async def start_recordings(
        self,
        app_id: str,
        *,
        account_id: str,
        allow_multiple_recordings: bool | Omit = omit,
        audio_config: recording_start_recordings_params.AudioConfig | Omit = omit,
        file_name_prefix: str | Omit = omit,
        interactive_config: recording_start_recordings_params.InteractiveConfig | Omit = omit,
        max_seconds: int | Omit = omit,
        meeting_id: str | Omit = omit,
        realtimekit_bucket_config: recording_start_recordings_params.RealtimekitBucketConfig | Omit = omit,
        rtmp_out_config: recording_start_recordings_params.RtmpOutConfig | Omit = omit,
        storage_config: Optional[recording_start_recordings_params.StorageConfig] | Omit = omit,
        url: str | Omit = omit,
        video_config: recording_start_recordings_params.VideoConfig | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecordingStartRecordingsResponse:
        """Starts recording a meeting.

        The meeting can be started by an App admin directly,
        or a participant with permissions to start a recording, based on the type of
        authorization used.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          allow_multiple_recordings: By default, a meeting allows only one recording to run at a time. Enabling the
              `allow_multiple_recordings` parameter to true allows you to initiate multiple
              recordings concurrently in the same meeting. This allows you to record separate
              videos of the same meeting with different configurations, such as portrait mode
              or landscape mode.

          audio_config: Object containing configuration regarding the audio that is being recorded.

          file_name_prefix: Update the recording file name.

          interactive_config: Allows you to add timed metadata to your recordings, which are digital markers
              inserted into a video file to provide contextual information at specific points
              in the content range. The ID3 tags containing this information are available to
              clients on the playback timeline in HLS format. The output files are generated
              in a compressed .tar format.

          max_seconds: Specifies the maximum duration for recording in seconds, ranging from a minimum
              of 60 seconds to a maximum of 24 hours.

          meeting_id: ID of the meeting to record.

          url: Pass a custom url to record arbitary screen

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/recordings", account_id=account_id, app_id=app_id
            ),
            body=await async_maybe_transform(
                {
                    "allow_multiple_recordings": allow_multiple_recordings,
                    "audio_config": audio_config,
                    "file_name_prefix": file_name_prefix,
                    "interactive_config": interactive_config,
                    "max_seconds": max_seconds,
                    "meeting_id": meeting_id,
                    "realtimekit_bucket_config": realtimekit_bucket_config,
                    "rtmp_out_config": rtmp_out_config,
                    "storage_config": storage_config,
                    "url": url,
                    "video_config": video_config,
                },
                recording_start_recordings_params.RecordingStartRecordingsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecordingStartRecordingsResponse,
        )

    async def start_track_recording(
        self,
        app_id: str,
        *,
        account_id: str,
        layers: Dict[str, recording_start_track_recording_params.Layers],
        meeting_id: str,
        max_seconds: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Starts a track recording in a meeting.

        Track recordings consist of "layers".
        Layers are used to map audio/video tracks in a meeting to output destinations.
        More information about track recordings is available in the
        [Track Recordings Guide Page](https://docs.realtime.cloudflare.com/guides/capabilities/recording/recording-overview).

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          meeting_id: ID of the meeting to record.

          max_seconds: Maximum seconds this recording should be active for (beta)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/recordings/track", account_id=account_id, app_id=app_id
            ),
            body=await async_maybe_transform(
                {
                    "layers": layers,
                    "meeting_id": meeting_id,
                    "max_seconds": max_seconds,
                },
                recording_start_track_recording_params.RecordingStartTrackRecordingParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class RecordingsResourceWithRawResponse:
    def __init__(self, recordings: RecordingsResource) -> None:
        self._recordings = recordings

        self.get_active_recordings = to_raw_response_wrapper(
            recordings.get_active_recordings,
        )
        self.get_one_recording = to_raw_response_wrapper(
            recordings.get_one_recording,
        )
        self.get_recordings = to_raw_response_wrapper(
            recordings.get_recordings,
        )
        self.pause_resume_stop_recording = to_raw_response_wrapper(
            recordings.pause_resume_stop_recording,
        )
        self.start_recordings = to_raw_response_wrapper(
            recordings.start_recordings,
        )
        self.start_track_recording = to_raw_response_wrapper(
            recordings.start_track_recording,
        )


class AsyncRecordingsResourceWithRawResponse:
    def __init__(self, recordings: AsyncRecordingsResource) -> None:
        self._recordings = recordings

        self.get_active_recordings = async_to_raw_response_wrapper(
            recordings.get_active_recordings,
        )
        self.get_one_recording = async_to_raw_response_wrapper(
            recordings.get_one_recording,
        )
        self.get_recordings = async_to_raw_response_wrapper(
            recordings.get_recordings,
        )
        self.pause_resume_stop_recording = async_to_raw_response_wrapper(
            recordings.pause_resume_stop_recording,
        )
        self.start_recordings = async_to_raw_response_wrapper(
            recordings.start_recordings,
        )
        self.start_track_recording = async_to_raw_response_wrapper(
            recordings.start_track_recording,
        )


class RecordingsResourceWithStreamingResponse:
    def __init__(self, recordings: RecordingsResource) -> None:
        self._recordings = recordings

        self.get_active_recordings = to_streamed_response_wrapper(
            recordings.get_active_recordings,
        )
        self.get_one_recording = to_streamed_response_wrapper(
            recordings.get_one_recording,
        )
        self.get_recordings = to_streamed_response_wrapper(
            recordings.get_recordings,
        )
        self.pause_resume_stop_recording = to_streamed_response_wrapper(
            recordings.pause_resume_stop_recording,
        )
        self.start_recordings = to_streamed_response_wrapper(
            recordings.start_recordings,
        )
        self.start_track_recording = to_streamed_response_wrapper(
            recordings.start_track_recording,
        )


class AsyncRecordingsResourceWithStreamingResponse:
    def __init__(self, recordings: AsyncRecordingsResource) -> None:
        self._recordings = recordings

        self.get_active_recordings = async_to_streamed_response_wrapper(
            recordings.get_active_recordings,
        )
        self.get_one_recording = async_to_streamed_response_wrapper(
            recordings.get_one_recording,
        )
        self.get_recordings = async_to_streamed_response_wrapper(
            recordings.get_recordings,
        )
        self.pause_resume_stop_recording = async_to_streamed_response_wrapper(
            recordings.pause_resume_stop_recording,
        )
        self.start_recordings = async_to_streamed_response_wrapper(
            recordings.start_recordings,
        )
        self.start_track_recording = async_to_streamed_response_wrapper(
            recordings.start_track_recording,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date, datetime
from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["ReportListParams"]


class ReportListParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    end: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """End of the time range (RFC3339). Takes precedence over to_date."""

    from_date: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """Deprecated, use `start` instead. Start date in YYYY-MM-DD format."""

    start: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Start of the time range (RFC3339). Takes precedence over from_date."""

    to_date: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """Deprecated, use `end` instead. End date in YYYY-MM-DD format."""
